@@ -22,9 +22,16 @@ import uuid
 from opentelemetry.trace import Status, StatusCode
 from sqlalchemy.event import listen
 
+from sqlalchemycollector.PlanCollectType import PlanCollectType
+
+METIS_DO_NOT_TRACK_COMMENT = '/*METIS_DO_NOT_TRACK*/'
+
 FILE_NAME = 'metis-log-collector.log'
 
 METIS_REQUEST_SPAN_ATTRIBUTE_IDENTIFIER = 'track.by.metis'
+METIS_PLAN_SPAN_ATTRIBUTE = 'metis.statement.plan'
+METIS_STATEMENT_SPAN_ATTRIBUTE = 'metis.statement'
+
 METIS_QUERY_SPAN_NAME = 'metis-query'
 
 
@@ -98,9 +105,10 @@ class MetisExporter(SpanExporter):
         data = {
             'logs': list(map(lambda x: {
                 '_uuid': str(uuid.uuid1()),
-                'query': x.attributes.get(SpanAttributes.DB_STATEMENT),
+                'query': x.attributes.get(METIS_STATEMENT_SPAN_ATTRIBUTE),
                 'dbEngine': x.attributes.get(SpanAttributes.DB_SYSTEM),
                 'date': datetime.utcnow().isoformat(),
+                'plan': json.loads(x.attributes.get(METIS_PLAN_SPAN_ATTRIBUTE)) if x.attributes.get(METIS_PLAN_SPAN_ATTRIBUTE) else None,
             }, metis_spans)),
             'framework': 'Flask',
             'path': parent.attributes.get(SpanAttributes.HTTP_TARGET, 'N/A'),
@@ -113,15 +121,15 @@ class MetisExporter(SpanExporter):
             file.write(json.dumps(data) + '\n')
 
 
-def collect_logs(app, engine, file_name=FILE_NAME):
+def collect_logs(app, engine, file_name=FILE_NAME, plan_collection_option=PlanCollectType.ESTIMATED):
     filename = os.getenv('METIS_LOG_FILE_NAME', file_name)
 
-    metis = MetisInstrumentor(filename)
+    metis = MetisInstrumentor(filename, plan_collection_option)
     metis.instrument_app(app, engine)
 
 
 class MetisInstrumentor:
-    def __init__(self, filename):
+    def __init__(self, filename, plan_collection_option):
         self.tracer_provider = TracerProvider()
         self.processor = SimpleSpanProcessor(MetisExporter(filename))
         self.tracer_provider.add_span_processor(self.processor)
@@ -130,6 +138,7 @@ class MetisInstrumentor:
             '',
             tracer_provider=self.tracer_provider,
         )
+        self.plan_collection_option = plan_collection_option
 
     def instrument_app(self, app, engine):
         def request_hook(span, flask_request_environ):
@@ -147,13 +156,27 @@ class MetisInstrumentor:
         db_vendor = _normalize_vendor(engine.name)
 
         def before_query_hook(conn, cursor, statement, parameters, context, executemany):
+            if statement.startswith(METIS_DO_NOT_TRACK_COMMENT):
+                return statement, parameters
+
             span = self.tracer.start_span(METIS_QUERY_SPAN_NAME, kind=trace.SpanKind.CLIENT)
 
             interpolated_statement = fix_sql_query(statement, parameters)
 
             span.set_attribute(SpanAttributes.DB_SYSTEM, db_vendor)
 
-            span.set_attribute(SpanAttributes.DB_STATEMENT, interpolated_statement)
+            span.set_attribute(METIS_STATEMENT_SPAN_ATTRIBUTE, interpolated_statement)
+
+            if self.plan_collection_option == PlanCollectType.ESTIMATED:
+                if conn.dialect.name == 'postgresql':
+                    with conn.connect() as connection:
+                        result = connection.execute(METIS_DO_NOT_TRACK_COMMENT + "explain (verbose, buffers, costs, summary, format JSON) " + statement, parameters)
+                        res = result.fetchall()
+                        if not res:
+                            raise Exception("No plan found")
+                        span.set_attribute(METIS_PLAN_SPAN_ATTRIBUTE, json.dumps(res[0][0][0]))
+                else:
+                    raise Exception("Plan collection is only supported for PostgreSQL")
 
             context._otel_span = span
             return statement, parameters
